unit Mailslot;

interface

uses
  forms,Windows, Messages, SysUtils, Classes, Graphics, Controls, Dialogs, ExtCtrls;

type
  TMsRole = (msClient,msServer);

type
  TMailSlot = class(TComponent)
  private
    { state and data buffers }
    FIsOpen   : Boolean;
    MsHandle  : THandle;
    { properties }
    FActive   : boolean;   { Open/Close state in boolean }
    FAlias    : string;    { name of mailslot alias }
    FServer   : string;    { mailslot path \\SERVER\mailslot\ALIAS }
    FMailslot : TMsRole;     { which role component has }
    FMsgSize  : Integer;  { max size of message }
    { methods }
    FOnOpen   : TNotifyEvent;
    FOnClose  : TNotifyEvent;
    { property read functions }
    function  GetServer:string;
    function  GetMessages:Integer;
    { property write functions }
    procedure SetServer(value:string);
    procedure SetActive(value:boolean);
    procedure SetAlias (value:string);
  protected
    { internal function - generic }
    function  Mailslotname:string;
    { internal functions - client part }
    procedure OpenClient;
    { internal functions - server part }
    procedure OpenServer;
  public
    { public declarations }
    procedure   Open;        { open mailslot as Client or Server }
    procedure   Close;       { close mailslot }
    procedure   Read (buffer:pointer;const size:word); { read from mailslot (only when server) }
    procedure   Write(buffer:pointer;const size:word); { write to mailslot (only when client) }

    constructor Create(AComponent:TComponent); override ;
    procedure   Free;
    property    Messages : Integer read GetMessages;
  published
    { properties }
    property Active  : boolean read FActive   write SetActive;
    property Server  : string  read GetServer write SetServer;
    property Mailslot: TMsRole   read FMailslot write FMailslot;
    property MsgSize : Integer read FMsgSize  write FMsgSize ;
    property Alias   : string  read FAlias    write SetAlias;
    { methods }
    property OnOpen  : TNotifyEvent read FOnOpen     write FOnOpen;
    property OnClose : TNotifyEvent read FOnClose    write FOnClose;
  end;


procedure Register;

implementation

const
  NotOpen   = 'Mailslot has not been opened';
  NotServer = 'Mailslot is not a server';
  NotClient = 'Mailslot is not a client';
  NotClosed = 'Mailslot is allready opened';
  FailClient= 'Could not create a mailslot client';
  FailServer= 'Could not create a mailslot server';
  FailWrite = 'Could not write msg to mailslot';
  FailRead  = 'Could not read msg from mailslot';
  FailClose = 'Could not close mailslot';

{ Internal tool functions and procedures }

function  TMailslot.Mailslotname:string;
begin
  mailslotname:=Server+'\mailslot\delphi\'+alias;
end;

{ mailslot server }

procedure TMailslot.OpenServer;
var
  temp : array[0..255] of char;
begin
  StrPCopy(Temp,mailslotname);
  MsHandle:=CreateMailslot(temp,MsgSize,1,NIL);
  FIsOpen:=(MsHandle<>INVALID_HANDLE_VALUE);
  If not FIsOpen then
    raise exception.create(FailServer)
  else
    if assigned(FOnOpen) then FOnOpen(SELF);
end;

{ mailslot client }
procedure TMailslot.OpenClient;
var
  Temp : array[0..255] of char;
begin
  StrPCopy(Temp,mailslotname);
  MsHandle:=CreateFile(
    temp,
    GENERIC_WRITE,
    FILE_SHARE_READ,
    NIL,
    OPEN_EXISTING,
    FILE_ATTRIBUTE_NORMAL,
    -1);
  FIsOpen:=MsHandle<>INVALID_HANDLE_VALUE;
  If not FIsOpen then
    raise exception.create(FailClient)
  else
    if assigned(FOnOpen) then FOnOpen(SELF);
end;

{ mailslot public functions }
procedure TMailslot.Open;
begin
  If not FIsOpen then case Mailslot of
    msClient : OpenClient;
    msServer : OpenServer;
  end else
    raise exception.create(NotClosed);
  FActive:=TRUE;
end;

procedure TMailslot.Close;
begin
  if FIsOpen then begin
    FIsOpen:=(Not CloseHandle(MsHandle));
    If FIsOpen then
      raise exception.Create(FailClose)
    else if Assigned(FOnClose) then FOnClose(SELF);
  end else
    raise exception.create(NotOpen);
  FActive:=False;
end;

procedure TMailslot.Read(buffer:pointer;const size:word);
var
  wfSize : Dword;
begin
  If FIsOpen then begin
    Case MailSlot of
      msServer : If not
       readFile(MsHandle,Buffer^,size,wfSize,NIL) then
         raise exception.create(FailRead);
      msClient : raise exception.create(NotServer);
    end;
  end else
    raise exception.create(NotOpen);
end;

procedure TMailslot.Write(buffer:pointer;const size:word);
var
 wfSize : Dword;
begin
  If FIsOpen then begin
    Case Mailslot of
      msClient : If not
       WriteFile(MsHandle,buffer^,Size,wfSize,nil) then
        raise exception.create(FailWrite);
      msServer : raise exception.create(NotClient);
    end;
  end else
    raise exception.create(NotOpen);
end;

{ property read }

function  TMailslot.GetMessages:Integer;
type
  pDWORD = ^DWORD;
var
  NextSize,
  MaxSize,
  MsgCount,
  TimeOut    : DWORD;
  pMaxSize,
  pMsgCount,
  pTimeOut   : pDWORD;
begin
  GetMessages:=0;
  if FIsOpen then case Mailslot of
    msServer : begin
      MaxSize :=FMsgSize; pMaxSize :=@MaxSize;
      TimeOut :=1;        pTimeOut :=@TimeOut;
      MsgCount:=0;        pMsgCount:=@MsgCount;
      NextSize:=0;
      If GetMailslotInfo(
        MsHandle,
        pMaxSize,
        NextSize,
        pMsgCount,
        pTimeOut) then GetMessages:=DWORD(pMsgCount^);
    end;
    msClient : raise exception.create(NotServer);
  end else
    raise exception.create(NotOpen);
end;

function  TMailslot.GetServer:string;
begin
  If Mailslot<>msServer then
    GetServer:=FServer
  else
    GetServer:='\\.';
end;

{ property write }

procedure TMailslot.SetAlias (value:string);
begin
  Active:=FALSE;
  FAlias:=value;
end;

procedure TMailslot.SetActive(Value:boolean);
begin
  case value of
    true : If not FIsOpen then Open;
    false: if FIsOpen then Close;
  end;
end;

procedure TMailslot.SetServer(Value:string);
var
  Size : Integer;
  Temp : Array[0..16] of char;
begin
  Active:=FALSE;
  If Value='' then begin
    Size:=SizeOf(Temp)-1;
    GetComputerName(@Temp,Size);
    Value:='\\'+StrPas(Temp);
  end;
  If Pos('\\',Value)=0 then
    Value:='\\'+Value;
  FServer:=UpperCase(Value);
end;

{ Init and Exit procedures etc }

constructor TMailslot.Create(AComponent:TComponent);
Var
  Temp : array[0..15] of char;
  Size : integer;
begin
  Size:=SizeOf(Temp);
  GetComputerName(@Temp,Size);
  inherited Create(AComponent);
  Server  :='\\'+StrPas(Temp);
  Mailslot:=msServer;
  MsHandle:=INVALID_HANDLE_VALUE;
  Alias   :=StrPas(Temp)+'_Mailslot';
  Active  :=false;
end;

procedure TMailSlot.Free;
begin
  If FIsOpen then Close;
  Inherited Free;
end;

{ registration procedure }

procedure Register;
begin
  RegisterComponents('Samples', [TMailslot]);
end;

end.
